package pp.monopoly.client.gui;

import com.jme3.math.FastMath;
import com.jme3.math.Vector3f;
import com.jme3.renderer.Camera;

/**
 * Steuert die Kamerabewegung in der Szene.
 */
public class CameraController {
    private final Camera camera;
    private final Vector3f center; // Fokuspunkt der Kamera
    private final float radius;    // Radius der Kreisbewegung
    private final float height;    // Höhe der Kamera über dem Spielfeld
    private final float speed;     // Geschwindigkeit der Kamerabewegung
    private float angle;           // Aktueller Winkel in der Kreisbewegung

    /**
     * Konstruktor für den CameraController.
     *
     * @param camera Die Kamera, die gesteuert werden soll
     * @param center Der Mittelpunkt der Kreisbewegung (Fokuspunkt)
     * @param radius Der Radius der Kreisbewegung
     * @param height Die Höhe der Kamera über dem Fokuspunkt
     * @param speed  Die Geschwindigkeit der Kamerabewegung
     */
    public CameraController(Camera camera, Vector3f center, float radius, float height, float speed) {
        this.camera = camera;
        this.center = center;
        this.radius = radius;
        this.height = height;
        this.speed = speed;
        this.angle = 0; // Starte bei Winkel 0
    }

    /**
     * Aktualisiert die Kameraposition und -ausrichtung.
     *
     * @param tpf Zeit pro Frame
     */
    public void update(float tpf) {
        camera.lookAt(center, Vector3f.UNIT_Y);
    }

    public void setPosition(int fieldID) {
        camera.setLocation(fieldIdToVector(fieldID));
    }

    public void setPosition(float x, float y) {
        camera.setLocation(new Vector3f(x,height,y));
    }

    private Vector3f fieldIdToVector(int fieldID) {
        if (fieldID <= 10) return new Vector3f(30,height,0);
        if (fieldID <= 20) return new Vector3f(0, height, 30);
        if (fieldID <= 30) return new Vector3f(-30, height, 0);
        if (fieldID <= 40) return new Vector3f(0, height, -30);
        else throw new IllegalArgumentException();
    }
}
