package pp.monopoly.client.gui;

import com.jme3.app.Application;
import com.jme3.app.state.BaseAppState;
import com.jme3.material.Material;
import com.jme3.math.ColorRGBA;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Quad;
import com.jme3.scene.shape.Sphere;
import com.jme3.texture.Texture;
import com.simsilica.lemur.Axis;
import com.simsilica.lemur.Button;
import com.simsilica.lemur.Container;
import com.simsilica.lemur.Insets3f;
import com.simsilica.lemur.Label;
import com.simsilica.lemur.Selector;
import com.simsilica.lemur.TextField;
import com.simsilica.lemur.component.QuadBackgroundComponent;
import com.simsilica.lemur.component.SpringGridLayout;

import com.simsilica.lemur.core.VersionedList;
import com.simsilica.lemur.core.VersionedReference;
import com.simsilica.lemur.style.ElementId;

import pp.dialog.Dialog;
import pp.monopoly.client.MonopolyApp;
import pp.monopoly.message.client.PlayerReady;

import java.util.Set;

public class LobbyMenu extends Dialog {

    private final MonopolyApp app;
    private final Container menuContainer;
    private Geometry background;
    private Geometry circle;
    private Container lowerLeftMenu;
    private Container lowerRightMenu;

    private TextField playerInputField = new TextField("Spieler 1");
    private TextField startingCapital = new TextField("15000");
    private String figure;

    public LobbyMenu(MonopolyApp app) {
        super(app.getDialogManager());
        this.app = app;

        // Hintergrundbild laden und hinzufügen
        addBackgroundImage();

        QuadBackgroundComponent translucentWhiteBackground =
                new QuadBackgroundComponent(new ColorRGBA(1.0f, 1.0f, 1.0f, 0.5f));

        menuContainer = new Container(new SpringGridLayout(Axis.Y, Axis.X));
        menuContainer.setPreferredSize(new Vector3f(1000, 600, 0)); // Fixed size of the container
        menuContainer.setBackground(translucentWhiteBackground);

        // Create a smaller horizontal container for the label, input field, and spacers
        Container horizontalContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.X, Axis.Y)));
        horizontalContainer.setPreferredSize(new Vector3f(600, 40, 0)); // Adjust container size
        horizontalContainer.setBackground(null);

        Label title = horizontalContainer.addChild(new Label("Startkapital:", new ElementId("label-Bold")));
        title.setFontSize(40);

        // Add a spacer between the title and the input field
        Label spacerBeforeInput = horizontalContainer.addChild(new Label("")); // Invisible spacer
        spacerBeforeInput.setPreferredSize(new Vector3f(20, 1, 0)); // Width of the spacer

        // Add an input field (TextField)
        horizontalContainer.addChild(startingCapital);
        startingCapital.setPreferredWidth(100); // Set the width of the input field
        startingCapital.setPreferredSize(new Vector3f(150, 50, 0));
        startingCapital.setInsets(new Insets3f(5, 10, 5, 10)); // Add padding around the text inside the field

        // Add a spacer after the input field
        Label spacerAfterInput = horizontalContainer.addChild(new Label("")); // Invisible spacer
        spacerAfterInput.setPreferredSize(new Vector3f(20, 1, 0)); // Width of the spacer

        menuContainer.setLocalTranslation(
                (app.getCamera().getWidth() - menuContainer.getPreferredSize().x) / 2,
                (app.getCamera().getHeight() + menuContainer.getPreferredSize().y) / 2,
                1
        );
        app.getGuiNode().attachChild(menuContainer);

        // Dropdowns and Labels
        Container dropdownContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.X, Axis.Y)));
        dropdownContainer.setPreferredSize(new Vector3f(800, 200, 0));
        dropdownContainer.setBackground(null);
        dropdownContainer.setInsets(new Insets3f(10, 0, 0, 0));
        // Player Input Field
        Container playerInputContainer = dropdownContainer.addChild(new Container(new SpringGridLayout(Axis.Y, Axis.X)));
        playerInputContainer.addChild(new Label("Spieler:"));
        playerInputContainer.setBackground(null);


        playerInputField.setPreferredSize(new Vector3f(100, 20, 0));
        playerInputField.setInsets(new Insets3f(5, 10, 5, 10)); // Add padding for the text inside the field
        playerInputField.setBackground(new QuadBackgroundComponent(ColorRGBA.Black));
        playerInputContainer.addChild(playerInputField);
        // Spacer (Center Circle Area)
        Label spacer = dropdownContainer.addChild(new Label(""));
        spacer.setPreferredSize(new Vector3f(200, 200, 0)); // Adjust this to fit the center graphic

        // Figur Dropdown
        Container figureDropdownContainer = dropdownContainer.addChild(new Container(new SpringGridLayout(Axis.Y, Axis.X)));
        figureDropdownContainer.addChild(new Label("Figur:"));
        figureDropdownContainer.setBackground(null);

        VersionedList<String> figures = new VersionedList<>();
        figures.add("Laptop");
        figures.add("Flugzeug");
        figures.add("Jägermeister");
        figures.add("Katze");
        figures.add("OOP");
        figures.add("Handyholster");

        Selector<String> figureDropdown = new Selector<>(figures, "glass");
        figureDropdown.setBackground(new QuadBackgroundComponent(ColorRGBA.DarkGray));
        figureDropdown.setPreferredSize(new Vector3f(100, 20, 0));
        figureDropdownContainer.addChild(figureDropdown);

        addSelectionActionListener(figureDropdown, this::onDropdownSelectionChanged);

        Container buttonContainer = menuContainer.addChild(new Container(new SpringGridLayout(Axis.X, Axis.Y)));
        buttonContainer.setPreferredSize(new Vector3f(100, 40, 0));
        buttonContainer.setInsets(new Insets3f(20, 0, 10, 0)); // Add spacing above the buttons
        buttonContainer.setBackground(null);
        // Lower-left container for "Abbrechen" button
        lowerLeftMenu = new Container();
        Button cancelButton = new Button("Abbrechen");
        cancelButton.setPreferredSize(new Vector3f(200, 60, 0)); // Set size to match the appearance in the image
        cancelButton.setFontSize(18); // Adjust font size
        cancelButton.addClickCommands(source -> close()); // Add functionality
        lowerLeftMenu.addChild(cancelButton);

        // Position the container near the bottom-left corner
        lowerLeftMenu.setLocalTranslation(new Vector3f(120, 170, 3)); // Adjust X and Y to align with the bottom-left corner
        app.getGuiNode().attachChild(lowerLeftMenu);

        // Lower-right container for "Bereit" button
        lowerRightMenu = new Container();
        Button readyButton = new Button("Bereit");
        readyButton.setPreferredSize(new Vector3f(200, 60, 0)); // Set size to match the appearance in the image
        readyButton.setFontSize(18); // Adjust font size
        readyButton.setBackground(new QuadBackgroundComponent(ColorRGBA.Green)); // Add color to match the style
        readyButton.addClickCommands(source -> toggleReady(null)); // Add functionality
        lowerRightMenu.addChild(readyButton);

        // Position the container near the bottom-right corner
        lowerRightMenu.setLocalTranslation(new Vector3f(app.getCamera().getWidth() - 320, 170, 3)); // X: 220px from the right, Y: 50px above the bottom
        app.getGuiNode().attachChild(lowerRightMenu);

        // Add a colored circle between the input field and the dropdown menu
        circle = createCircle( ColorRGBA.Red); // 50 is the diameter, Red is the color
        circle.setLocalTranslation(new Vector3f(
                (app.getCamera().getWidth()) / 2, // Center horizontally
                (app.getCamera().getHeight() / 2) - 90, // Adjust Y position
                2 // Ensure it's in front of the background but behind the dropdown
        ));
        app.getGuiNode().attachChild(circle); // Attach to the GUI node

        // Zentrierung des Containers
        menuContainer.setLocalTranslation(
                (app.getCamera().getWidth() - menuContainer.getPreferredSize().x) / 2,
                (app.getCamera().getHeight() + menuContainer.getPreferredSize().y) / 2,
                1  // Höhere Z-Ebene für den Vordergrund
        );

        app.getGuiNode().attachChild(menuContainer);
    }


    /**
     * Lädt das Hintergrundbild und fügt es als geometrische Ebene hinzu.
     */
    private void addBackgroundImage() {
        Texture backgroundImage = app.getAssetManager().loadTexture("Pictures/lobby.png");
        Quad quad = new Quad(app.getCamera().getWidth(), app.getCamera().getHeight());
        background = new Geometry("Background", quad);
        Material backgroundMaterial = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        backgroundMaterial.setTexture("ColorMap", backgroundImage);
        background.setMaterial(backgroundMaterial);
        background.setLocalTranslation(0, 0, -1); // Hintergrundebene

        app.getGuiNode().attachChild(background);
    }

    private Geometry createCircle(ColorRGBA color) {

        Sphere sphere = new Sphere(90,90,60.0f);
        Geometry circleGeometry = new Geometry("Circle", sphere);

        // Create a material with a solid color
        Material material = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        material.setColor("Color", color); // Set the desired color
        circleGeometry.setMaterial(material);

        return circleGeometry;
    }

    /**
     * Schaltet den "Bereit"-Status um.
     */
    private void toggleReady(Label playersLabel) {
        app.getGameLogic().send(new PlayerReady(true, playerInputField.getText(), figure, Integer.parseInt(startingCapital.getText())));
    }

    @Override
    public void escape() {
        super.close();
    }

    /**
     * Adds a custom action listener to the Selector.
     */
    private void addSelectionActionListener(Selector<String> selector, SelectionActionListener<String> listener) {
        VersionedReference<Set<Integer>> selectionRef = selector.getSelectionModel().createReference();

        app.getStateManager().attach(new BaseAppState() {
            @Override
            public void update(float tpf) {
                if (selectionRef.update()) {
                    String selected = selectionRef.get().toString();
                    System.out.println(selected);
                    listener.onSelectionChanged(selected);
                }
            }

            @Override
            protected void initialize(Application app) {
            }

            @Override
            protected void cleanup(Application app) {
            }

            @Override
            protected void onEnable() {

            }

            @Override
            protected void onDisable() {

            }
        });
    }

    /**
     * Callback for when the dropdown selection changes.
     */
    private void onDropdownSelectionChanged(String selected) {
        System.out.println("Selected: " + selected);
        switch (selected) {
            case "[0]":
                figure = "Laptop";
                break;
            case "[1]":
                figure = "Flugzeug";
                break;
            case "[2]":
                figure = "Jägermeister";
                break;
            case "[3]":
                figure = "Katze";
                break;
            case "[4]":
                figure = "OOP";
                break;
            case "[5]":
                figure = "Handyholster";
                break;
            default:
                break;    
        }
    }

    /**
     * Functional interface for a selection action listener.
     */
    @FunctionalInterface
    private interface SelectionActionListener<T> {
        void onSelectionChanged(T selection);
    }
}
