////////////////////////////////////////
// Programming project code
// UniBw M, 2022, 2023, 2024
// www.unibw.de/inf2
// (c) Mark Minas (mark.minas@unibw.de)
////////////////////////////////////////

package pp.monopoly.client.gui;

import java.util.prefs.Preferences;

import com.simsilica.lemur.Button;
import com.simsilica.lemur.Checkbox;
import com.simsilica.lemur.Label;
import com.simsilica.lemur.style.ElementId;

import static pp.monopoly.Resources.lookup;
import pp.monopoly.client.GameMusic;
import pp.monopoly.client.GameSound;
import pp.monopoly.client.MonopolyApp;
import pp.dialog.Dialog;
import pp.dialog.StateCheckboxModel;
import static pp.util.PreferencesUtils.getPreferences;

/**
 * The Menu class represents the main menu in the Battleship game application.
 * It extends the Dialog class and provides functionalities for loading, saving,
 * returning to the game, and quitting the application.
 */
public class SettingsMenu extends Dialog {
    private static final Preferences PREFERENCES = getPreferences(SettingsMenu.class);
    private static final String LAST_PATH = "last.file.path";
    private final MonopolyApp app;
    private final VolumeSlider musicSlider;
    private final SoundSlider soundSlider;

    /**
     * Constructs the Menu dialog for the Battleship application.
     *
     * @param app the MonopolyApp instance
     */
    public SettingsMenu(MonopolyApp app) {
        super(app.getDialogManager());
        this.app = app;
        musicSlider = new VolumeSlider(app.getStateManager().getState(GameMusic.class));
        soundSlider = new SoundSlider(app.getStateManager().getState(GameSound.class));
        addChild(new Label("Einstellungen", new ElementId("settings-title"))); //NON-NLS
        addChild(new Label("Sound Effekte", new ElementId("label"))); //NON-NLS

        addChild(soundSlider);

        addChild(new Checkbox("Soundeffekte an / aus", new StateCheckboxModel(app, GameSound.class)));
        
        addChild(new Label("Hintergrund Musik", new ElementId("label"))); //NON-NLS
        addChild(new Checkbox("Musik an / aus", new StateCheckboxModel(app, GameMusic.class)));
        
        addChild(musicSlider);

        addChild(new Button("Zurück zum Spiel", new ElementId("button"))).addClickCommands(s -> ifTopDialog(this::close));
        addChild(new Button("Beenden", new ElementId("button"))).addClickCommands(s -> ifTopDialog(app::closeApp));
        
        update();
    }

    /**
     * Updates the state of the load and save buttons based on the game logic.
     */
    @Override
    public void update() {
    }

    @Override
    public void update(float delta) {
        musicSlider.update();
        soundSlider.update();
    }

    /**
     * As an escape action, this method closes the menu if it is the top dialog.
     */
    @Override
    public void escape() {
        close();
    }
}
