package pp.monopoly.client.gui;

import com.jme3.material.Material;
import com.jme3.math.Vector3f;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Quad;
import com.jme3.texture.Texture;
import com.simsilica.lemur.Axis;
import com.simsilica.lemur.Button;
import com.simsilica.lemur.Container;
import com.simsilica.lemur.HAlignment;
import com.simsilica.lemur.component.QuadBackgroundComponent;
import com.simsilica.lemur.component.SpringGridLayout;

import pp.dialog.Dialog;
import pp.monopoly.client.MonopolyApp;

/**
 * Constructs the startup menu dialog for the Monopoly application.
import pp.monopoly.client.gui.GameMenu;
 */
public class StartMenu extends Dialog {
    private final MonopolyApp app;
    private Container logoContainer;
    private Container unibwLogoContainer;

    /**
     * Constructs the Startup Menu dialog for the Monopoly application.
     *
     * @param app the MonopolyApp instance
     */
    public StartMenu(MonopolyApp app) {
        super(app.getDialogManager());
        this.app = app;
        
        int screenWidth = app.getContext().getSettings().getWidth();
        int screenHeight = app.getContext().getSettings().getHeight();

        // Set up the background image
        Texture backgroundImage = app.getAssetManager().loadTexture("Pictures/unibw-Bib2.png");
        Quad quad = new Quad(screenWidth, screenHeight);
        Geometry background = new Geometry("Background", quad);
        Material backgroundMaterial = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        backgroundMaterial.setTexture("ColorMap", backgroundImage);
        background.setMaterial(backgroundMaterial);
        background.setLocalTranslation(0, 0, -1); // Ensure it is behind other GUI elements
        app.getGuiNode().attachChild(background);

        // Center container for title and play button
        Container centerMenu = new Container(new SpringGridLayout(Axis.Y, Axis.X));

        Button startButton = new Button("Spielen");
        startButton.setPreferredSize(new Vector3f(190, 60, 0)); // Increase button size (width, height)
        startButton.setFontSize(40); // Set the font size for the button text
        startButton.setTextHAlignment(HAlignment.Center); // Center the text horizontally

        startButton.addClickCommands(s -> app.connect());
        centerMenu.addChild(startButton);

        // Position the center container in the middle of the screen
        centerMenu.setLocalTranslation(new Vector3f(screenWidth / 2f - centerMenu.getPreferredSize().x / 2f,
                                                    screenHeight / 2f - 280 + centerMenu.getPreferredSize().y / 2f,
                                                    0));
        app.getGuiNode().attachChild(centerMenu);

        // Load the Monopoly logo as a texture
        Texture logoTexture = app.getAssetManager().loadTexture("Pictures/logo-monopoly.png");

        // Create a container for the logo
        Container logoContainer = new Container();
        QuadBackgroundComponent logoBackground = new QuadBackgroundComponent(logoTexture);
        logoContainer.setBackground(logoBackground);

        // Set the size of the container to fit the logo
        float logoWidth = 512;  // Adjust these values based on the logo dimensions
        float logoHeight = 128; // Adjust these values based on the logo dimensions
        logoContainer.setPreferredSize(new Vector3f(logoWidth, logoHeight, 0));

        // Position the container at the center of the screen
        logoContainer.setLocalTranslation(new Vector3f(
                screenWidth / 2f - logoWidth / 2f,
                screenHeight / 2f + 200, // Adjust this value for vertical position
                0
        ));

        // Attach the container to the GUI node
        app.getGuiNode().attachChild(logoContainer);

        // Load the Unibw logo as a texture
        Texture unibwTexture = app.getAssetManager().loadTexture("Pictures/logo-unibw.png");

        // Create a container for the Unibw logo
        Container unibwContainer = new Container();
        QuadBackgroundComponent unibwBackground = new QuadBackgroundComponent(unibwTexture);
        unibwContainer.setBackground(unibwBackground);

        // Set the size of the container to fit the Unibw logo
        float unibwWidth = 512;  // Adjust these values based on the logo dimensions
        float unibwHeight = 128; // Adjust these values based on the logo dimensions
        unibwContainer.setPreferredSize(new Vector3f(unibwWidth, unibwHeight, 0));

        // Position the container slightly below the Monopoly logo
        unibwContainer.setLocalTranslation(new Vector3f(
                screenWidth / 2f - unibwWidth / 2f,
                screenHeight / 2f + 100, // Adjust this value for vertical position
                0
        ));

        // Attach the container to the GUI node
        app.getGuiNode().attachChild(unibwContainer);
    }

    @Override
    public void escape() {
        close();
    }
}