package pp.monopoly.client.gui.popups;

import com.jme3.material.Material;
import com.jme3.material.RenderState.BlendMode;
import com.jme3.math.ColorRGBA;
import com.jme3.scene.Geometry;
import com.jme3.scene.shape.Quad;
import com.simsilica.lemur.Container;
import com.simsilica.lemur.Label;
import com.simsilica.lemur.component.QuadBackgroundComponent;
import com.simsilica.lemur.style.ElementId;
import pp.dialog.Dialog;
import pp.monopoly.client.MonopolyApp;
import pp.monopoly.model.fields.GateField;

/**
 * SettingsMenu ist ein Overlay-Menü, das durch ESC aufgerufen werden kann.
 */
public class GateFieldCard extends Dialog {
    private final MonopolyApp app;
    private final Geometry overlayBackground;
    private final Container gateFieldContainer;
    private final Container backgroundContainer;
    private int index = 5;

    public GateFieldCard(MonopolyApp app) {
        super(app.getDialogManager());
        this.app = app;

        //Generate the corresponfing field
        GateField field = (GateField) app.getGameLogic().getBoardManager().getFieldAtIndex(index);

        // Halbtransparentes Overlay hinzufügen
        overlayBackground = createOverlayBackground();
        app.getGuiNode().attachChild(overlayBackground);

        // Create the background container
        backgroundContainer = new Container();
        backgroundContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.8657f, 0.8735f, 0.8892f, 1.0f))); // Darker background
        app.getGuiNode().attachChild(backgroundContainer);

        // Hauptcontainer für die Gebäudekarte
        gateFieldContainer = new Container();
        gateFieldContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.8657f, 0.8735f, 0.8892f, 1.0f)));

        // Titel
        // Die Namen werden dynamisch dem BoardManager entnommen
        Label settingsTitle = gateFieldContainer.addChild(new Label(field.getName(), new ElementId("settings-title")));
        settingsTitle.setFontSize(48);
        settingsTitle.setColor(ColorRGBA.Black);

        // Text, der auf der Karte steht
        // Die Preise werden dynamisch dem BoardManager entnommen
        Container propertyValuesContainer = gateFieldContainer.addChild(new Container());
        propertyValuesContainer.addChild(new Label("„Preis: " + field.getPrice() + " EUR", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("Wenn man 1 Bahnhof besitzt: 250 EUR", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("Wenn man 2 Bahnhöfe besitzt: 500 EUR", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("Wenn man 3 Bahnhöfe besitzt: 1000 EUR", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("Wenn man 4 Bahnhöfe besitzt: 2000 EUR", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("", new ElementId("label-Text")));
        propertyValuesContainer.addChild(new Label("„Hypothek: " + field.getHypo() + " EUR", new ElementId("label-Text")));
        propertyValuesContainer.setBackground(new QuadBackgroundComponent(new ColorRGBA(0.4657f, 0.4735f, 0.4892f, 1.0f)));

        //TODO eventuell diese Stelle löschen, da nur die BuyCard Kaufen und beenden hat

        /*
        // Beenden-Button
        Button quitButton = foodFieldContainer.addChild(new Button("Beenden", new ElementId("button")));
        quitButton.setFontSize(32);
        // Kaufen-Button
        Button buyButton = foodFieldContainer.addChild(new Button("Kaufen", new ElementId("button")));
        buyButton.setFontSize(32);
         */

        float padding = 10; // Padding around the settingsContainer for the background
        backgroundContainer.setPreferredSize(gateFieldContainer.getPreferredSize().addLocal(padding, padding, 0));


        // Zentriere das Menü
        gateFieldContainer.setLocalTranslation(
            (app.getCamera().getWidth() - gateFieldContainer.getPreferredSize().x) / 2,
            (app.getCamera().getHeight() + gateFieldContainer.getPreferredSize().y) / 2,
            8
        );

        backgroundContainer.setLocalTranslation(
                (app.getCamera().getWidth() - gateFieldContainer.getPreferredSize().x - padding) / 2,
                (app.getCamera().getHeight() + gateFieldContainer.getPreferredSize().y+ padding) / 2,
                7
        );

        app.getGuiNode().attachChild(gateFieldContainer);
    }

    /**
     * Erstellt einen halbtransparenten Hintergrund für das Menü.
     *
     * @return Geometrie des Overlays
     */
    private Geometry createOverlayBackground() {
        Quad quad = new Quad(app.getCamera().getWidth(), app.getCamera().getHeight());
        Geometry overlay = new Geometry("Overlay", quad);
        Material material = new Material(app.getAssetManager(), "Common/MatDefs/Misc/Unshaded.j3md");
        material.setColor("Color", new ColorRGBA(0, 0, 0, 0.5f)); // Halbtransparent
        material.getAdditionalRenderState().setBlendMode(BlendMode.Alpha);
        overlay.setMaterial(material);
        overlay.setLocalTranslation(0, 0, 0);
        return overlay;
    }

    /**
     * Schließt das Menü und entfernt die GUI-Elemente.
     */
    @Override
    public void close() {
        System.out.println("Schließe SettingsMenu..."); // Debugging-Ausgabe
        app.getGuiNode().detachChild(gateFieldContainer);  // Entferne das Menü
        app.getGuiNode().detachChild(backgroundContainer); //Entfernt Rand
        app.getGuiNode().detachChild(overlayBackground);  // Entferne das Overlay
        // app.setSettingsMenuOpen(false);                  // Menü als geschlossen markieren
        // app.unblockInputs();                             // Eingaben wieder aktivieren
        System.out.println("SettingsMenu geschlossen."); // Debugging-Ausgabe
    }

    public void setIndex(int index) {
        this.index = index;
    }
}
