package pp.monopoly.message.client;

import com.jme3.network.serializing.Serializable;

/**
 * Represents a message indicating the player is ready to play.
 */
@Serializable
public class PlayerReady extends ClientMessage {
    private boolean isReady;
    private String name;
    private String figure;
    private int startMoney;

    /**
     * Default constructor for serialization purposes.
     */
    private PlayerReady() { /* empty */ }

    /**
     * Constructs a PlayerReady message.
     *
     * @param isReady true if the player is ready, false otherwise
     * @param name    the name of the player
     * @param color   the color of the player (can be null)
     */
    public PlayerReady(boolean isReady, String name, String figure, int startMoney) {
        this.isReady = isReady;
        this.name = name;
        this.figure = figure;
        this.startMoney = startMoney;
    }

    public String getName() {
        return name;
    }

    public String getFigure() {
        return figure;
    }

    public boolean isReady() {
        return isReady;
    }

    public int getStartMoney() {
        return startMoney;
    }

    @Override
    public void accept(ClientInterpreter interpreter, int from) {
        interpreter.received(this, from);
    }
}
